import csv
import argparse
import functools
from tabulate import tabulate

PAR=2
TIMEOUT=600.0

itp0_solved = 0
itp0_time = 0.0
itp0_uniqes = 0
itp0_parscore = 0.0

itp2_solved = 0
itp2_time = 0.0
itp2_uniqes = 0
itp2_parscore = 0.0

itp4_solved = 0
itp4_time = 0.0
itp4_uniqes = 0
itp4_parscore = 0.0

itp5_solved = 0
itp5_time = 0.0
itp5_uniqes = 0
itp5_parscore = 0.0

osmt_port_02_solved = 0
osmt_port_02_time = 0
osmt_port_02_parscore = 0.0

osmt_port_04_solved = 0
osmt_port_04_time = 0
osmt_port_04_parscore = 0.0

osmt_port_24_solved = 0
osmt_port_24_time = 0
osmt_port_24_parscore = 0.0

osmt_port_024_solved = 0
osmt_port_024_time = 0
osmt_port_024_parscore = 0.0

msat0_solved = 0
msat0_time = 0.0
msat0_uniqes = 0
msat0_parscore = 0.0

msat1_solved = 0
msat1_time = 0.0
msat1_uniqes = 0
msat1_parscore = 0.0

msat2_solved = 0
msat2_time = 0.0
msat2_uniqes = 0
msat2_parscore = 0.0

msat_port_01_solved = 0
msat_port_01_parscore = 0.0

msat_port_02_solved = 0
msat_port_02_parscore = 0.0

msat_port_12_solved = 0
msat_port_12_parscore = 0.0

msat_port_012_solved = 0
msat_port_012_parscore = 0.0

def run(path):
    with open(path, newline='') as csvfile:
        reader = csv.reader(csvfile)
        for row in reader:
            process(row)


    print(tabulate([
        ['OSMT_port_02', osmt_port_02_solved, osmt_port_02_time, osmt_port_02_parscore]
        ,['OSMT_port_04', osmt_port_04_solved, osmt_port_04_time, osmt_port_04_parscore]
        ,['OSMT_port_24', osmt_port_24_solved, osmt_port_24_time, osmt_port_24_parscore]
        ,['OSMT_port_024', osmt_port_024_solved, osmt_port_024_time, osmt_port_024_parscore]
        ], headers=['solved', 'time', 'par-2']))
    
    print()

    itp0_regret = osmt_port_24_parscore - osmt_port_024_parscore
    itp2_regret = osmt_port_04_parscore - osmt_port_024_parscore
    itp4_regret = osmt_port_02_parscore - osmt_port_024_parscore
    
    print(tabulate([
        ['ITP_F', osmt_port_024_solved - osmt_port_24_solved, itp0_regret, itp0_regret / (osmt_port_024_parscore / 100.0)]
        ,['ITP_DF', osmt_port_024_solved - osmt_port_04_solved, itp2_regret, itp2_regret / (osmt_port_024_parscore / 100.0)] 
        ,['ITP_Dec', osmt_port_024_solved - osmt_port_02_solved, itp4_regret, itp4_regret / (osmt_port_024_parscore / 100.0)]
        ], headers=['uniquely_solved', 'osmt_port_024_regret', 'regret in %']))

    print()


    print(tabulate([
        ['msat_port_01', msat_port_01_solved, msat_port_01_parscore]
        ,['msat_port_02', msat_port_02_solved, msat_port_02_parscore]
        ,['msat_port_12', msat_port_12_solved, msat_port_12_parscore]
        ,['msat_port_012', msat_port_012_solved, msat_port_012_parscore]
        ], headers=['solved', 'par-2']))
    
    print()

    print(tabulate([
        ['Itp_F', msat0_solved, msat0_time, msat0_parscore]
        ,['Itp_M', msat1_solved, msat1_time, msat1_parscore]
        ,['Itp_DF', msat2_solved, msat2_time, msat2_parscore]
        ], headers=['solved', 'time', 'par-2']))
    
    print()

    msat0_regret = msat_port_12_parscore - msat_port_012_parscore
    msat1_regret = msat_port_02_parscore - msat_port_012_parscore
    msat2_regret = msat_port_01_parscore - msat_port_012_parscore
    
    print(tabulate([
        ['ITP_F', msat_port_012_solved - msat_port_12_solved, msat0_regret, msat0_regret / (msat_port_012_parscore / 100.0)]
        ,['ITP_M', msat_port_012_solved - msat_port_02_solved, msat1_regret, msat1_regret / (msat_port_012_parscore / 100.0)] 
        ,['ITP_DF', msat_port_012_solved - msat_port_01_solved, msat2_regret, msat2_regret / (msat_port_012_parscore / 100.0)]
        ], headers=['uniquely_solved', 'msat_port_012_regret', 'regret in %']))

    print()


def issolved(val):
    return val == 'valid' or val == 'invalid'

def process(row):
    bench = row[0]
    itp0_t = float(row[3])
    itp0_r = row[4]
    itp2_t = float(row[7])
    itp2_r = row[8]
    itp4_t = float(row[11])
    itp4_r = row[12]
    itp5_t = float(row[19])
    itp5_r = row[20]
    msat0_t = float(row[27])
    msat0_r = row[28]
    msat1_t = float(row[31])
    msat1_r = row[32]
    msat2_t = float(row[35])
    msat2_r = row[36]
    #print(itp0_t, itp0_r, itp2_t, itp2_r, itp4_t, itp4_r, itp5_t, itp5_r, msat0_t, msat0_r, msat1_t, msat1_r, msat2_t, msat2_r)
    results = [itp0_r, itp2_r, itp4_r, itp5_r, msat0_r, msat1_r, msat2_r]
    times = [itp0_t, itp2_t, itp4_t, itp5_t, msat0_t, msat1_t, msat2_t]
    # sanitycheck that no inconsistencies occured
    if len(list(set([res for res in results if issolved(res)]))) >= 2:
        print(row)
    assert ( len(list(set([res for res in results if issolved(res)]))) < 2)
    osmt_port_02_t = min([times[i] for i in [0,1] if issolved(results[i])], default=TIMEOUT)
    osmt_port_02_r = functools.reduce(lambda a,b : a if issolved(a) else b, [itp0_r, itp2_r])
    osmt_port_04_t = min([times[i] for i in [0,2] if issolved(results[i])], default=TIMEOUT)
    osmt_port_04_r = functools.reduce(lambda a,b : a if issolved(a) else b, [itp0_r, itp4_r])
    osmt_port_24_t = min([times[i] for i in [1,2] if issolved(results[i])], default=TIMEOUT)
    osmt_port_24_r = functools.reduce(lambda a,b : a if issolved(a) else b, [itp2_r, itp4_r])
    osmt_port_024_t = min([times[i] for i in [0,1,2] if issolved(results[i])], default=TIMEOUT)
    osmt_port_024_r = functools.reduce(lambda a,b : a if issolved(a) else b, [itp0_r, itp2_r, itp4_r])

    global itp0_solved
    global itp2_solved
    global itp4_solved
    global itp5_solved
    global itp0_time
    global itp2_time
    global itp4_time
    global itp5_time
    global itp0_uniqes
    global itp2_uniqes
    global itp4_uniqes
    global itp5_uniqes
    global itp0_parscore
    global itp2_parscore
    global itp4_parscore
    global itp5_parscore

    global osmt_port_02_solved
    global osmt_port_02_time
    global osmt_port_02_parscore
    
    global osmt_port_04_solved
    global osmt_port_04_time
    global osmt_port_04_parscore
    
    global osmt_port_24_solved
    global osmt_port_24_time
    global osmt_port_24_parscore
    
    global osmt_port_024_solved
    global osmt_port_024_time
    global osmt_port_024_parscore

    itp0_solved = itp0_solved + 1 if issolved(itp0_r) else itp0_solved
    itp2_solved = itp2_solved + 1 if issolved(itp2_r) else itp2_solved
    itp4_solved = itp4_solved + 1 if issolved(itp4_r) else itp4_solved
    itp5_solved = itp5_solved + 1 if issolved(itp5_r) else itp5_solved
    osmt_port_024_solved = osmt_port_024_solved + 1 if issolved(osmt_port_024_r) else osmt_port_024_solved
    osmt_port_02_solved = osmt_port_02_solved + 1 if issolved(osmt_port_02_r) else osmt_port_02_solved
    osmt_port_04_solved = osmt_port_04_solved + 1 if issolved(osmt_port_04_r) else osmt_port_04_solved
    osmt_port_24_solved = osmt_port_24_solved + 1 if issolved(osmt_port_24_r) else osmt_port_24_solved

    itp0_time = itp0_time + itp0_t if issolved(itp0_r) else itp0_time
    itp2_time = itp2_time + itp2_t if issolved(itp2_r) else itp2_time
    itp4_time = itp4_time + itp4_t if issolved(itp4_r) else itp4_time
    itp5_time = itp5_time + itp5_t if issolved(itp5_r) else itp5_time
    osmt_port_02_time= osmt_port_02_time + osmt_port_02_t if issolved(osmt_port_02_r) else osmt_port_02_time
    osmt_port_04_time= osmt_port_04_time + osmt_port_04_t if issolved(osmt_port_04_r) else osmt_port_04_time
    osmt_port_24_time= osmt_port_24_time + osmt_port_24_t if issolved(osmt_port_24_r) else osmt_port_24_time
    osmt_port_024_time= osmt_port_024_time + osmt_port_024_t if issolved(osmt_port_024_r) else osmt_port_024_time

    solved_indices = [x for x in [0,1,2,3] if issolved(results[x])]
    if len(solved_indices) == 1:
        i = solved_indices[0]
        if i == 0:
            itp0_uniqes = itp0_uniqes + 1
        elif i == 1:
            itp2_uniqes = itp2_uniqes + 1
        elif i == 2:
            itp4_uniqes = itp4_uniqes + 1
        elif i == 3:
            itp5_uniqes = itp5_uniqes + 1

    itp0_parscore = itp0_parscore + itp0_t if issolved(itp0_r) else itp0_parscore + PAR*TIMEOUT
    itp2_parscore = itp2_parscore + itp2_t if issolved(itp2_r) else itp2_parscore + PAR*TIMEOUT
    itp4_parscore = itp4_parscore + itp4_t if issolved(itp4_r) else itp4_parscore + PAR*TIMEOUT
    itp5_parscore = itp5_parscore + itp5_t if issolved(itp5_r) else itp5_parscore + PAR*TIMEOUT
    osmt_port_02_parscore = osmt_port_02_parscore + osmt_port_02_t if issolved(osmt_port_02_r) else osmt_port_02_parscore + PAR*TIMEOUT
    osmt_port_04_parscore = osmt_port_04_parscore + osmt_port_04_t if issolved(osmt_port_04_r) else osmt_port_04_parscore + PAR*TIMEOUT
    osmt_port_24_parscore = osmt_port_24_parscore + osmt_port_24_t if issolved(osmt_port_24_r) else osmt_port_24_parscore + PAR*TIMEOUT
    osmt_port_024_parscore = osmt_port_024_parscore + osmt_port_024_t if issolved(osmt_port_024_r) else osmt_port_024_parscore + PAR*TIMEOUT

    global msat0_solved
    global msat0_time
    global msat0_parscore
    
    global msat1_solved
    global msat1_time
    global msat1_parscore
    
    global msat2_solved
    global msat2_time
    global msat2_parscore

    global msat_port_01_solved
    global msat_port_01_parscore
    
    global msat_port_02_solved
    global msat_port_02_parscore
    
    global msat_port_12_solved
    global msat_port_12_parscore
    
    global msat_port_012_solved
    global msat_port_012_parscore

    
    msat_port_01_t = min([times[i] for i in [4,5] if issolved(results[i])], default=TIMEOUT)
    msat_port_01_r = functools.reduce(lambda a,b : a if issolved(a) else b, [msat0_r, msat1_r])
    msat_port_02_t = min([times[i] for i in [4,6] if issolved(results[i])], default=TIMEOUT)
    msat_port_02_r = functools.reduce(lambda a,b : a if issolved(a) else b, [msat0_r, msat2_r])
    msat_port_12_t = min([times[i] for i in [5,6] if issolved(results[i])], default=TIMEOUT)
    msat_port_12_r = functools.reduce(lambda a,b : a if issolved(a) else b, [msat1_r, msat2_r])
    msat_port_012_t = min([times[i] for i in [4,5,6] if issolved(results[i])], default=TIMEOUT)
    msat_port_012_r = functools.reduce(lambda a,b : a if issolved(a) else b, [msat0_r, msat1_r, msat2_r])

    msat0_solved = msat0_solved + 1 if issolved(msat0_r) else msat0_solved
    msat1_solved = msat1_solved + 1 if issolved(msat1_r) else msat1_solved
    msat2_solved = msat2_solved + 1 if issolved(msat2_r) else msat2_solved
    msat_port_012_solved = msat_port_012_solved + 1 if issolved(msat_port_012_r) else msat_port_012_solved
    msat_port_01_solved = msat_port_01_solved + 1 if issolved(msat_port_01_r) else msat_port_01_solved
    msat_port_02_solved = msat_port_02_solved + 1 if issolved(msat_port_02_r) else msat_port_02_solved
    msat_port_12_solved = msat_port_12_solved + 1 if issolved(msat_port_12_r) else msat_port_12_solved

    msat0_time = msat0_time + msat0_t if issolved(msat0_r) else msat0_time
    msat1_time = msat1_time + msat1_t if issolved(msat1_r) else msat1_time
    msat2_time = msat2_time + msat2_t if issolved(msat2_r) else msat2_time


    msat0_parscore = msat0_parscore + msat0_t if issolved(msat0_r) else msat0_parscore + PAR*TIMEOUT
    msat1_parscore = msat1_parscore + msat1_t if issolved(msat1_r) else msat1_parscore + PAR*TIMEOUT
    msat2_parscore = msat2_parscore + msat2_t if issolved(msat2_r) else msat2_parscore + PAR*TIMEOUT
    msat_port_01_parscore = msat_port_01_parscore + msat_port_01_t if issolved(msat_port_01_r) else msat_port_01_parscore + PAR*TIMEOUT
    msat_port_02_parscore = msat_port_02_parscore + msat_port_02_t if issolved(msat_port_02_r) else msat_port_02_parscore + PAR*TIMEOUT
    msat_port_12_parscore = msat_port_12_parscore + msat_port_12_t if issolved(msat_port_12_r) else msat_port_12_parscore + PAR*TIMEOUT
    msat_port_012_parscore = msat_port_012_parscore + msat_port_012_t if issolved(msat_port_012_r) else msat_port_012_parscore + PAR*TIMEOUT
    

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("file", help=".csv file to process")
    args = parser.parse_args()
    run(args.file)

